require 'kramdown'
require 'oga'

Jekyll::Hooks.register :site, :pre_render do |site|
    categories = {}
    catcounts = {}
    tagcounts = {}
    links = [] 

    # We iterate over the posts instead of the categories.  That ensures
    # that we will only add categories that actually have posts.
    site.posts.docs.each_with_index do |post, i|
        lang = post.data['lang']
        categories[lang] = {} unless categories[lang]
        catcounts[lang] = {} unless catcounts[lang]

        for catname in post.data['categories']
            if catcounts[lang][catname]
                catcounts[lang][catname] = catcounts[lang][catname] + 1
            else
                catcounts[lang][catname] = 1
            end
            categories[lang][catname] = site.config['t'][lang]['catnames'][catname]
        end

        tagcounts[lang] = {} unless tagcounts[lang]
        for tag in post.data['tags']
            if tagcounts[lang][tag]
                tagcounts[lang][tag] = tagcounts[lang][tag] + 1
            else
                tagcounts[lang][tag] = 1
            end
        end

        # Extract all links.
        html = Kramdown::Document.new(post.content).to_html

        # Weed out raw sections as they may outsmart Oga.
        html = html.gsub(/{%[ ]*raw[ ]*%}(.|\n)*?{%[ ]*raw[ ]*%}/, '')
        doc = Oga.parse_html(html)
        links[i] = {}
        doc.css('a').each do |anchor|
            href = anchor.get('href')
            if href && href[0] != '#'
                links[i][href] = true
            end
        end
    end

    # Iterate over all posts again, and assign similarity points.
    similarity = []
    site.posts.docs.each_with_index do |post, i|
        similarity[i] = []
        site.posts.docs.take(i).each_with_index do |other, j|
            similarity[i][j] = 0

            if post.data['lang'] == other.data['lang']
                # 1 point for each shared link.
                links[i].each do |href, value|
                    if links[j].include? href
                        similarity[i][j] += 1
                    end
                end

                # 2 points for each shared tag.
                for tag in post.data['tags']
                    if other.data['tags'].include? tag
                        similarity[i][j] += 2
                    end
                end

                # 3 points for each shared category.
                for cat in post.data['categories']
                    if other.data['categories'].include? cat
                        similarity[i][j] += 3 
                    end
                end
                
                # 5 points if the two documents are linked.
                post_href = '/' + post.data['slug'] + '/'
                other_href = '/' + other.data['slug'] + '/'
                if links[i].include? other_href
                    similarity[i][j] += 5
                elsif links[j].include? post_href
                    similarity[i][j] += 5
                end
            end
        end
    end 
    
    # Fill the missing similarity values.
    for i in 0 .. site.posts.docs.length - 1
        for j in 0 .. i
            if i == j
                similarity[j][i] = 0
            else
                similarity[j][i] = similarity[i][j]
            end
        end
    end

    # Now fill the matrix
    site.config['related'] = {}
    for i in 0 .. site.posts.docs.length - 1
        post = site.posts.docs[i]
        similarities = similarity[i]
        hash = {}
        for j in 0 .. site.posts.docs.length - 1
            other = site.posts.docs[j]
            if i != j && post.data['lang'] == other.data['lang']
                path = '/' + site.posts.docs[j].data['slug'] + '/'
                hash[j] = similarities[j]
            end
        end
        sorted = Hash[hash.sort_by{ |k, v| v }].keys.reverse
        site.config['related'][post.id] = sorted.map! { |idx| 
            site.posts.docs[idx] 
        }.take(10)
    end

    site.config['catcounts'] = catcounts
    site.config['tagcounts'] = tagcounts
    
    site.config['tagnames'] = {}
    for lang in tagcounts.keys
        site.config['tagnames'][lang] = tagcounts[lang].keys
    end
   
    site.config['sorted-categories'] = {} 
    for lang in categories.keys
        other = categories[lang].delete('other')
        site.config['sorted-categories'][lang] = 
            Hash[categories[lang].sort_by{|k, v| v}]
        if other
            site.config['sorted-categories'][lang]['other'] = 
                site.config['t'][lang]['catnames']['other']
        end
    end
end
